@@ -255,7 +255,7 @@ void MagnifierEffect::zoomIn()
         polling = true;
         effects->startMousePolling();
     }
-    if (!m_texture) {
+    if (effects->isOpenGLCompositing() && !m_texture) {
         m_texture = new GLTexture(magnifier_size.width(), magnifier_size.height());
         m_texture->setYInverted(false);
         m_fbo = new GLRenderTarget(*m_texture);
@@ -266,7 +266,7 @@ void MagnifierEffect::zoomIn()
 void MagnifierEffect::zoomOut()
 {
     target_zoom /= 1.2;
-    if (target_zoom < 1) {
+    if (target_zoom <= 1) {
         target_zoom = 1;
         if (polling) {
             polling = false;
@@ -293,7 +293,7 @@ void MagnifierEffect::toggle()
             polling = true;
             effects->startMousePolling();
         }
-        if (!m_texture) {
+        if (effects->isOpenGLCompositing() && !m_texture) {
             m_texture = new GLTexture(magnifier_size.width(), magnifier_size.height());
             m_texture->setYInverted(false);
             m_fbo = new GLRenderTarget(*m_texture);
